/*
fuzzy_ts.cc

This C++ file defines member functions for the class FUZZY_TS_MODULE
It was generated with rcsdesign
with template version 1.1

MODIFICATIONS:
Fri Dec 15 15:49:32 EST 2000	Modified with RCS-Design tool.
Thu Dec 14 20:30:48 EST 2000	Created with RCS-Design tool.

*/

// Include Files
#include "rcs.hh" 	// Common RCS definitions
#include "nml_mod.hh" 	// NML_MODULE definitions
#include "fuzzy_ts.hh" 	// FUZZY_TS_MODULE definitions
// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.


#include "fuzzy_tsn.hh" 	// NML Commands and Status definitions for fuzzy_ts
// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


// Constructor
FUZZY_TS_MODULE::FUZZY_TS_MODULE(int _is_base_class)
{

	// Initialize the NML channels if this module is not being used as the base class for another module.
	if(!_is_base_class)
	{
		INITIALIZE_NML();
	}
	// Add additional code to initialize the module here.
	num_of_inputs = NUM_OF_INPUTS;
	num_of_ts_inputs = NUM_OF_TS_INPUTS;

	fuzzy_ts_sys = new FUZZY_TS_SYS(num_of_inputs, num_of_ts_inputs);

	if(fuzzy_ts_sys->fuzzy_ts_sys_err != FUZZY_TS_SYS_NO_ERROR)
	{
		logError("Error in creating FUZZY_TS_MODULE object!!!");
		status = RCS_ERROR; // let my supervisor know I have a problem.
	}

}

// Overloaded Virtual Functions

/*
INITIALIZE_NML

The INITIALIZE_NML function is ussually called only once from within the
constructor. It should not be called if a derived class will also call it.

*/
void FUZZY_TS_MODULE::INITIALIZE_NML()
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	setErrorLogChannel(new NML(nmlErrorFormat, "errlog", "fuzzy_ts", "fuzzy_ts_module.nml"));
	setCmdChannel(new RCS_CMD_CHANNEL(fuzzy_tsFormat, "fuzzy_ts_cmd", "fuzzy_ts", "fuzzy_ts_module.nml"));
	fuzzy_ts_status = new FUZZY_TS_STATUS();
	setStatChannel(new RCS_STAT_CHANNEL(fuzzy_tsFormat, "fuzzy_ts_sts", "fuzzy_ts", "fuzzy_ts_module.nml"), fuzzy_ts_status);
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


}

/*
PRE_PROCESS

The PRE_PROCESS function is called every cycle after the command and
subordinates status have been read but before DECISION_PROCESS is called.
It is intended to be used for tasks such as sensory processing that should
be performed every cycle regardless of the current command or state.

*/
void FUZZY_TS_MODULE::PRE_PROCESS()
{
  // Pre-Processing Code
  // Read and/or calculate inputs to the TS fuzzy system 
  // Here we just assign them to zero

  // System inputs
  for(int ii = 0; ii < num_of_inputs; ii++)
    {
      sys_inputs[ii] = 0.0;
    }

  // TS inputs
  for(int ii = 0; ii < num_of_ts_inputs; ii++)
    {
      ts_inputs[ii] = 0.0;
    }

  // Read (or calculate) the error from the system
  // error = 
}

/*
  DECISION_PROCESS
  
The DECISION_PROCESS function is called every cycle as long as there is a non-zero command.
It is expected to call a command function based on commandInData->type.

*/
void FUZZY_TS_MODULE::DECISION_PROCESS()
{
	switch(commandInData->type)
	{
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	case FUZZY_TS_INIT_TYPE:
		INIT((FUZZY_TS_INIT *)commandInData);
		break;

	case FUZZY_TS_HALT_TYPE:
		HALT((FUZZY_TS_HALT *)commandInData);
		break;

	case FUZZY_TS_RUN_TYPE:
		RUN((FUZZY_TS_RUN *)commandInData);
		break;

	case FUZZY_TS_SET_ADAPTATION_GAIN_TYPE:
		SET_ADAPTATION_GAIN((FUZZY_TS_SET_ADAPTATION_GAIN *)commandInData);
		break;

	case FUZZY_TS_SET_INPUT_GAINS_TYPE:
		SET_INPUT_GAINS((FUZZY_TS_SET_INPUT_GAINS *)commandInData);
		break;

	case FUZZY_TS_SET_IN_MF_TYPE:
		SET_IN_MF((FUZZY_TS_SET_IN_MF *)commandInData);
		break;

	case FUZZY_TS_SET_IN_UD_TYPE:
		SET_IN_UD((FUZZY_TS_SET_IN_UD *)commandInData);
		break;

	case FUZZY_TS_SET_PREMISE_TYPE:
		SET_PREMISE((FUZZY_TS_SET_PREMISE *)commandInData);
		break;

	case FUZZY_TS_SET_TS_COEFF_TYPE:
		SET_TS_COEFF((FUZZY_TS_SET_TS_COEFF *)commandInData);
		break;
		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	default:
		logError("The command %d is not recognized.",commandInData->type);
		break;
	}
}

/*
POST_PROCESS

The POST_PROCESS function is called every cycle after DECISION_PROCESS is called
but before the status and the subordinates commands  have been written.
It is intended to be used for tasks such as output filters that should
be performed every cycle regardless of the current command or state.

*/
void FUZZY_TS_MODULE::POST_PROCESS()
{
	// Post-Processing Code
	static int input_ud_index = 0;
	static int input_mf_index = 0;
	static int rule_index = 0;
	int mf_index;

	// Report the current system inputs
	memcpy(fuzzy_ts_status->sys_inputs, sys_inputs, 
		num_of_inputs*sizeof(double));

	// Report the current TS inputs
	memcpy(fuzzy_ts_status->ts_inputs, ts_inputs, 
		num_of_ts_inputs*sizeof(double));

	// Report the current system output
	fuzzy_ts_status->fuzzy_ts_output = fuzzy_ts_output;

	// Report the current premise type used
	fuzzy_ts_status->premise_type = fuzzy_ts_sys->get_premise_type();

	// Report the current adaptation gain used
	fuzzy_ts_status->adapatation_gain = fuzzy_ts_sys->get_eta();

	// Input Report

	// If done with this UD switch to the next one
	if (input_mf_index == fuzzy_ts_sys->in_ud[input_mf_index]->get_num_of_mfs())
	{
		input_mf_index = 0;  
		input_ud_index++;
	}

	// If done with all UD start from first
	if (input_ud_index == num_of_inputs)
	{
		input_ud_index = 0;
	}

	// Each Cycle time report status of the current MF in the current UD
	fuzzy_ts_status->num_of_current_in_ud = input_ud_index;
	fuzzy_ts_status->gain_for_in_ud = fuzzy_ts_sys->in_ud[input_ud_index]->ud_gain;

	fuzzy_ts_status->num_of_current_in_mf = input_mf_index;
	fuzzy_ts_status->type_of_current_in_mf = 
	fuzzy_ts_sys->in_ud[input_ud_index]->mfs[input_mf_index].get_type();
	fuzzy_ts_status->center_of_current_in_mf = 
	fuzzy_ts_sys->in_ud[input_ud_index]->mfs[input_mf_index].get_center();
	fuzzy_ts_status->spread_of_current_in_mf = 
	fuzzy_ts_sys->in_ud[input_ud_index]->mfs[input_mf_index].get_spread();

	// Output Parameters Report

	fuzzy_ts_status->num_of_current_rule = rule_index;

	for(int ii = 0; ii <= num_of_ts_inputs; ii++)
	  {
	    fuzzy_ts_status->ts_coeff[ii] = 
	      fuzzy_ts_sys->ts_coeff->get_element(rule_index, ii);
	  }
	
	// Move to the next rule on the next iteration
	rule_index++;

	// If done with all the rules strat from the first rule
	if (rule_index == fuzzy_ts_sys->get_num_of_rules())
	{
		rule_index = 0;  
	}

}

// Command Functions

/*
INIT

Parameter(s):
FUZZY_TS_INIT *cmd_in -- NML Message sent from superior.

Most Modules will have an INIT command.
The INIT function is expected to initialize any variables that may be
in an uninitialized or unknown state, send INIT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.

*/
void FUZZY_TS_MODULE::INIT(FUZZY_TS_INIT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		// Send an INIT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


		stateNext(S1);
		// Reinitialize variables here.

	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
HALT

Parameter(s):
FUZZY_TS_HALT *cmd_in -- NML Message sent from superior.

Most Modules will have an HALT command.
The HALT function is expected to stop any motion or command execution,
send HALT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.

*/
void FUZZY_TS_MODULE::HALT(FUZZY_TS_HALT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		//Send a HALT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

		stateNext(S1);
	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
RUN
SET_ADAPTATION_GAIN

Parameter(s):
FUZZY_TS_RUN *cmd_in -- NML Message sent from superior.
FUZZY_TS_SET_ADAPTATION_GAIN *cmd_in -- NML Message sent from superior.

*/
void FUZZY_TS_MODULE::RUN(FUZZY_TS_RUN *cmd_in)
{
  // Put state table for FUZZY_TS_RUN here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      // Calculate the system output
      fuzzy_ts_output = fuzzy_ts_sys->control(sys_inputs, ts_inputs);

      // Update output parameters if necessary
      // Should read the error from PRE_PROCESS function
      //      fuzzy_ts_sys->update_ts_coeff(error);

    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

void FUZZY_TS_MODULE::SET_ADAPTATION_GAIN(FUZZY_TS_SET_ADAPTATION_GAIN *cmd_in)
{
	// Put state table for FUZZY_TS_SET_ADAPTATION_GAIN here.
	if(STATE_MATCH(NEW_COMMAND))
	{
		status = RCS_EXEC;
		stateNext(S1);
	}
	else if(STATE_MATCH(S1))
	{
		fuzzy_ts_sys->set_eta(cmd_in->eta);
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
SET_INPUT_GAINS

Parameter(s):
FUZZY_TS_SET_INPUT_GAINS *cmd_in -- NML Message sent from superior.

*/
void FUZZY_TS_MODULE::SET_INPUT_GAINS(FUZZY_TS_SET_INPUT_GAINS *cmd_in)
{
	// Put state table for FUZZY_TS_SET_INPUT_GAINS here.
	if(STATE_MATCH(NEW_COMMAND))
	{
		status = RCS_EXEC;
		stateNext(S1);
	}
	else if(STATE_MATCH(S1))
	{
		fuzzy_ts_sys->set_input_gains(cmd_in->input_gains);
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
SET_IN_MF

Parameter(s):
FUZZY_TS_SET_IN_MF *cmd_in -- NML Message sent from superior.

*/
void FUZZY_TS_MODULE::SET_IN_MF(FUZZY_TS_SET_IN_MF *cmd_in)
{
	// Put state table for FUZZY_TS_SET_IN_MF here.
	int ii, jj;
	 
	if(STATE_MATCH(NEW_COMMAND))
	{
		status = RCS_EXEC;
		stateNext(S1);
	}
	else if(STATE_MATCH(S1))
	{
		ii = cmd_in->num_of_current_in_ud;
		jj = cmd_in->num_of_current_in_mf;
		 
		fuzzy_ts_sys->in_ud[ii]->mfs[jj].set_type(cmd_in->mf_type);
		fuzzy_ts_sys->in_ud[ii]->mfs[jj].set_center(cmd_in->mf_center);
		fuzzy_ts_sys->in_ud[ii]->mfs[jj].set_spread(cmd_in->mf_spread);

		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
SET_IN_UD

Parameter(s):
FUZZY_TS_SET_IN_UD *cmd_in -- NML Message sent from superior.

*/
void FUZZY_TS_MODULE::SET_IN_UD(FUZZY_TS_SET_IN_UD *cmd_in)
{
	// Put state table for FUZZY_TS_SET_IN_UD here.
	int ii;
	 
	if(STATE_MATCH(NEW_COMMAND))
	{
		status = RCS_EXEC;
		stateNext(S1);
	}
	else if(STATE_MATCH(S1))
	{
		ii = cmd_in->num_of_current_in_ud;
		 
		fuzzy_ts_sys->in_ud[ii]->ud_gain = cmd_in->gain;
		fuzzy_ts_sys->in_ud[ii]->init_ud(cmd_in->first_center, 
			cmd_in->last_center, 
			cmd_in->mf_type, 
			cmd_in->spread);
		 
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
SET_PREMISE

Parameter(s):
FUZZY_TS_SET_PREMISE *cmd_in -- NML Message sent from superior.

*/
void FUZZY_TS_MODULE::SET_PREMISE(FUZZY_TS_SET_PREMISE *cmd_in)
{
	// Put state table for FUZZY_TS_SET_PREMISE here.
	if(STATE_MATCH(NEW_COMMAND))
	{
		status = RCS_EXEC;
		stateNext(S1);
	}
	else if(STATE_MATCH(S1))
	{
		fuzzy_ts_sys->set_premise_type(cmd_in->premise_type);
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
SET_TS_COEFF

Parameter(s):
FUZZY_TS_SET_TS_COEFF *cmd_in -- NML Message sent from superior.

*/
void FUZZY_TS_MODULE::SET_TS_COEFF(FUZZY_TS_SET_TS_COEFF *cmd_in)
{
	// Put state table for FUZZY_TS_SET_TS_COEFF here.
	// Put state table for FUZZY_TS_SET_PREMISE here.
	if(STATE_MATCH(NEW_COMMAND))
	{
		status = RCS_EXEC;
		stateNext(S1);
	}
	else if(STATE_MATCH(S1))
	{
	  // Set the all the TS coefficients for the specified rule 
	  for(int ii = 0; ii <= num_of_ts_inputs; ii++)
	    {
	      fuzzy_ts_sys->ts_coeff->set_element(cmd_in->num_of_current_rule, ii, 
						  cmd_in->ts_coeff[ii]);
	    }
	  status = RCS_DONE;
	  stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

