 /*
  * FILE --- veh_sim_fuzzy_ts.hh
  *
  * PURPOSE --- To provide an example use of the class for 4'th
  * order Runge-Kutta simulation, RK4_SIM, and the class for implementing
  * n-input 1-output Takagi-Sugeno (TS) fuzzy system, FUZZY_TS_SYS.
  *
  * Implements the member functions of a class for performing a simulation of
  * a simple vehicle with point mass dynamics controlled by a TS fuzzy controller.
  * The fuzzy controller is implemented as a part of the class for the vehicle
  * dynamics - VEH_SIM_FUZZY_TS. 
  *
  */

// Prevent Multiple Inclusion
#ifndef VEH_SIM_FUZZY_TS_HH
#define VEH_SIM_FUZZY_TS_HH

// Include Files
#include <math.h>
#include "rk4_sim.hh"
#include "fuzzy_ts_sys.hh"

class VEH_SIM_FUZZY_TS : public RK4_SIM
{
public:
  VEH_SIM_FUZZY_TS (double samp_time = 0.005, int _simulation_per_cycle = 10,
		    double _initial_time = 0.0, double *_initial_state = NULL);
  ~VEH_SIM_FUZZY_TS ();
  
  
  FUZZY_TS_SYS *fuzzy_ts_sys;  // The TS fuzzy controller
  double sampling_period;
  int simulation_per_cycle; // # of RK4 simulations per sampling interval
  
  // Overload DERIV function of RK4_SIM for simulating vehicle dynamics
  inline void DERIV(const double t, const double *x, 
		    const double *u, double *xdot);
  
  // the Simulation routine
  void simulate ();
  
  double output_gain;

private:
  // Plant parameters
  double mass;
  double drag_x;
  double tau;
  double disturbance;
};

#endif // VEH_SIM_FUZZY_TS_HH
